@@ -37,6 +37,7 @@
 #endif
 
 extern unsigned int flags;
+extern unsigned int dm_major;
 extern int  dis;
 extern char timestamp[][TIMESTAMP_LEN];
 extern unsigned long avg_count;
@@ -171,6 +172,8 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 			else if (DISPLAY_CPU_ALL(a->opt_flags)) {
 				printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f"
 				       "    %6.2f    %6.2f    %6.2f\n",
+				       (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
+				       0.0 :
 				       ll_sp_value(scp->cpu_user - scp->cpu_guest,
 						   scc->cpu_user - scc->cpu_guest,     g_itv),
 				       ll_sp_value(scp->cpu_nice,    scc->cpu_nice,    g_itv),
@@ -387,10 +390,12 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 		*smc = (struct stats_memory *) a->buf[curr],
 		*smp = (struct stats_memory *) a->buf[prev];
 	static unsigned long long
-		avg_frmkb = 0,
-		avg_bufkb = 0,
-		avg_camkb = 0,
-		avg_comkb = 0;
+		avg_frmkb    = 0,
+		avg_bufkb    = 0,
+		avg_camkb    = 0,
+		avg_comkb    = 0,
+		avg_activekb = 0,
+		avg_inactkb  = 0;
 	static unsigned long long
 		avg_frskb = 0,
 		avg_tlskb = 0,
@@ -411,12 +416,12 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 	if (DISPLAY_MEM_AMT(a->opt_flags)) {
 		if (dis) {
 			printf("\n%-11s kbmemfree kbmemused  %%memused kbbuffers  kbcached"
-			       "  kbcommit   %%commit\n", timestamp[!curr]);
+			       "  kbcommit   %%commit  kbactive   kbinact\n", timestamp[!curr]);
 		}
 
 		if (!dispavg) {
 			/* Display instantaneous values */
-			printf("%-11s %9lu %9lu    %6.2f %9lu %9lu %9lu   %7.2f\n",
+			printf("%-11s %9lu %9lu    %6.2f %9lu %9lu %9lu   %7.2f %9lu %9lu\n",
 			       timestamp[curr],
 			       smc->frmkb,
 			       smc->tlmkb - smc->frmkb,
@@ -426,21 +431,25 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 			       smc->camkb,
 			       smc->comkb,
 			       (smc->tlmkb + smc->tlskb) ?
-			       SP_VALUE(0, smc->comkb, smc->tlmkb + smc->tlskb) : 0.0);
+			       SP_VALUE(0, smc->comkb, smc->tlmkb + smc->tlskb) : 0.0,
+			       smc->activekb,
+			       smc->inactkb);
 
 			/*
 			 * Will be used to compute the average.
 			 * We assume that the total amount of memory installed can not vary
 			 * during the interval given on the command line.
 			 */
-			avg_frmkb += smc->frmkb;
-			avg_bufkb += smc->bufkb;
-			avg_camkb += smc->camkb;
-			avg_comkb += smc->comkb;
+			avg_frmkb    += smc->frmkb;
+			avg_bufkb    += smc->bufkb;
+			avg_camkb    += smc->camkb;
+			avg_comkb    += smc->comkb;
+			avg_activekb += smc->activekb;
+			avg_inactkb  += smc->inactkb;
 		}
 		else {
 			/* Display average values */
-			printf("%-11s %9.0f %9.0f    %6.2f %9.0f %9.0f %9.0f   %7.2f\n",
+			printf("%-11s %9.0f %9.0f    %6.2f %9.0f %9.0f %9.0f   %7.2f %9.0f %9.0f\n",
 			       timestamp[curr],
 			       (double) avg_frmkb / avg_count,
 			       (double) smc->tlmkb - ((double) avg_frmkb / avg_count),
@@ -453,11 +462,13 @@ void stub_print_memory_stats(struct activity *a, int prev, int curr,
 			       (double) avg_comkb / avg_count,
 			       (smc->tlmkb + smc->tlskb) ?
 			       SP_VALUE(0.0, (double) (avg_comkb / avg_count),
-					smc->tlmkb + smc->tlskb) :
-			       0.0);
+					smc->tlmkb + smc->tlskb) : 0.0,
+			       (double) avg_activekb / avg_count,
+			       (double) avg_inactkb / avg_count);
 			
 			/* Reset average counters */
 			avg_frmkb = avg_bufkb = avg_camkb = avg_comkb = 0;
+			avg_activekb = avg_inactkb = 0;
 		}
 	}
 	
@@ -657,45 +668,50 @@ void stub_print_queue_stats(struct activity *a, int prev, int curr, int dispavg)
 	struct stats_queue
 		*sqc = (struct stats_queue *) a->buf[curr];
 	static unsigned long long
-		avg_nr_running  = 0,
-		avg_nr_threads  = 0,
-		avg_load_avg_1  = 0,
-		avg_load_avg_5  = 0,
-		avg_load_avg_15 = 0;
+		avg_nr_running    = 0,
+		avg_nr_threads    = 0,
+		avg_load_avg_1    = 0,
+		avg_load_avg_5    = 0,
+		avg_load_avg_15   = 0,
+		avg_procs_blocked = 0;
 	
 	if (dis) {
-		printf("\n%-11s   runq-sz  plist-sz   ldavg-1   ldavg-5  ldavg-15\n",
+		printf("\n%-11s   runq-sz  plist-sz   ldavg-1   ldavg-5  ldavg-15   blocked\n",
 		       timestamp[!curr]);
 	}
 
 	if (!dispavg) {
 		/* Display instantaneous values */
-		printf("%-11s %9lu %9u %9.2f %9.2f %9.2f\n", timestamp[curr],
+		printf("%-11s %9lu %9u %9.2f %9.2f %9.2f %9lu\n", timestamp[curr],
 		       sqc->nr_running,
 		       sqc->nr_threads,
 		       (double) sqc->load_avg_1  / 100,
 		       (double) sqc->load_avg_5  / 100,
-		       (double) sqc->load_avg_15 / 100);
+		       (double) sqc->load_avg_15 / 100,
+		       sqc->procs_blocked);
 
 		/* Will be used to compute the average */
-		avg_nr_running  += sqc->nr_running;
-		avg_nr_threads  += sqc->nr_threads;
-		avg_load_avg_1  += sqc->load_avg_1;
-		avg_load_avg_5  += sqc->load_avg_5;
-		avg_load_avg_15 += sqc->load_avg_15;
+		avg_nr_running    += sqc->nr_running;
+		avg_nr_threads    += sqc->nr_threads;
+		avg_load_avg_1    += sqc->load_avg_1;
+		avg_load_avg_5    += sqc->load_avg_5;
+		avg_load_avg_15   += sqc->load_avg_15;
+		avg_procs_blocked += sqc->procs_blocked;
 	}
 	else {
 		/* Display average values */
-		printf("%-11s %9.0f %9.0f %9.2f %9.2f %9.2f\n", timestamp[curr],
-		       (double) avg_nr_running  / avg_count,
-		       (double) avg_nr_threads  / avg_count,
-		       (double) avg_load_avg_1  / (avg_count * 100),
-		       (double) avg_load_avg_5  / (avg_count * 100),
-		       (double) avg_load_avg_15 / (avg_count * 100));
+		printf("%-11s %9.0f %9.0f %9.2f %9.2f %9.2f %9.0f\n", timestamp[curr],
+		       (double) avg_nr_running    / avg_count,
+		       (double) avg_nr_threads    / avg_count,
+		       (double) avg_load_avg_1    / (avg_count * 100),
+		       (double) avg_load_avg_5    / (avg_count * 100),
+		       (double) avg_load_avg_15   / (avg_count * 100),
+		       (double) avg_procs_blocked / avg_count);
 
 		/* Reset average counters */
 		avg_nr_running = avg_nr_threads = 0;
 		avg_load_avg_1 = avg_load_avg_5 = avg_load_avg_15 = 0;
+		avg_procs_blocked = 0;
 	}
 }
 
@@ -821,7 +837,7 @@ __print_funct_t print_disk_stats(struct activity *a, int prev, int curr,
 		
 		dev_name = NULL;
 
-		if ((USE_PRETTY_OPTION(flags)) && (sdc->major == DEVMAP_MAJOR)) {
+		if ((USE_PRETTY_OPTION(flags)) && (sdc->major == dm_major)) {
 			dev_name = transform_devmapname(sdc->major, sdc->minor);
 		}
 
