@@ -38,6 +38,7 @@
 static char *seps[] =  {"\t", ";"};
 
 extern unsigned int flags;
+extern unsigned int dm_major;
 
 /*
  ***************************************************************************
@@ -190,6 +191,8 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 					render(isdb, pre, PT_NOFLAG,
 					       "all\t%%usr", "-1", NULL,
 					       NOVAL,
+					       (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
+					       0.0 :
 					       ll_sp_value(scp->cpu_user - scp->cpu_guest,
 							   scc->cpu_user - scc->cpu_guest,
 							   g_itv));
@@ -291,7 +294,8 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 					render(isdb, pre, PT_NOFLAG,
 					       "cpu%d\t%%usr", "%d", cons(iv, i - 1, NOVAL),
 					       NOVAL,
-					       !g_itv ?
+					       (!g_itv ||
+					       ((scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest))) ?
 					       0.0 :			/* CPU is offline or tickless */
 					       ll_sp_value(scp->cpu_user - scp->cpu_guest,
 							   scc->cpu_user - scc->cpu_guest, g_itv));
@@ -684,11 +688,19 @@ __print_funct_t render_memory_stats(struct activity *a, int isdb, char *pre,
 		       "-\tkbcommit", NULL, NULL,
 		       smc->comkb, DNOVAL);
 
-		render(isdb, pre, pt_newlin,
+		render(isdb, pre, PT_NOFLAG,
 		       "-\t%%commit", NULL, NULL, NOVAL,
 		       (smc->tlmkb + smc->tlskb) ?
 		       SP_VALUE(0, smc->comkb, smc->tlmkb + smc->tlskb) :
 		       0.0);
+
+		render(isdb, pre, PT_USEINT,
+		       "-\tkbactive", NULL, NULL,
+		       smc->activekb, DNOVAL);
+
+		render(isdb, pre, PT_USEINT | pt_newlin,
+		       "-\tkbinact", NULL, NULL,
+		       smc->inactkb, DNOVAL);
 	}
 	
 	if (DISPLAY_SWAP(a->opt_flags)) {
@@ -794,10 +806,14 @@ __print_funct_t render_queue_stats(struct activity *a, int isdb, char *pre,
 	       NOVAL,
 	       (double) sqc->load_avg_5 / 100);
 
-	render(isdb, pre, pt_newlin,
+	render(isdb, pre, PT_NOFLAG,
 	       "-\tldavg-15", NULL, NULL,
 	       NOVAL,
 	       (double) sqc->load_avg_15 / 100);
+	
+	render(isdb, pre, PT_USEINT | pt_newlin,
+	       "-\tblocked", NULL, NULL,
+	       sqc->procs_blocked, DNOVAL);
 }
 
 /*
@@ -906,7 +922,7 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 
 		dev_name = NULL;
 
-		if ((USE_PRETTY_OPTION(flags)) && (sdc->major == DEVMAP_MAJOR)) {
+		if ((USE_PRETTY_OPTION(flags)) && (sdc->major == dm_major)) {
 			dev_name = transform_devmapname(sdc->major, sdc->minor);
 		}
 
